"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.configureStore = void 0;
const react_1 = require("react");
const mobx_devtools_mst_1 = __importDefault(require("mobx-devtools-mst"));
/*
 * Generic Types:
 *  - first generic is GenRootStoreModel, which extends Instance type from MST. This
 *  is needed for methods, that Instance provides.
 *  - second generic is GenStoreEnv, which extends object for now. This is needed for TS to recognize
 *  stores that exist on the store bus
 *  - third generic is GenInstantiatedStoresBox, which represents instantiated stores
 *  that are loaded into the RootStore.
 *  - fourth generic is GenBus, which represent instantiated stores
 *  that must be accessible through the store bus
 */
const configureStore = ({ RootStore, InstantiatedStoresBox, Bus, ExecutionEnv = "production", }) => {
    // for now it will be a function with its own scope
    const createStore = () => {
        const env = {
            ...Bus,
        };
        const rootStore = RootStore.create({
            ...InstantiatedStoresBox,
        }, env);
        return rootStore;
    };
    const rootStore = createStore();
    // React utils
    const StoreContext = (0, react_1.createContext)(null);
    const StoreProvider = ({ children, customStore }) => {
        if (!children) {
            throw new Error("Provider cannot be mounted without React Element inside");
        }
        if (ExecutionEnv === "development") {
            (0, mobx_devtools_mst_1.default)(rootStore);
        }
        return (0, react_1.createElement)(StoreContext.Provider, {
            value: customStore || rootStore,
        }, children);
    };
    const useStore = () => {
        const value = (0, react_1.useContext)(StoreContext);
        if (!value) {
            throw new Error("useStore must be used within a StoreProvider. Please, add context!");
        }
        return value;
    };
    const storeStruct = {
        rootStore,
        StoreContext,
        StoreProvider,
        useStore,
    };
    return storeStruct;
};
exports.configureStore = configureStore;
